import 'dart:io';
import 'dart:convert';
import 'package:args/command_runner.dart';
import 'package:cli_util/cli_logging.dart';
import 'package:string_scanner/string_scanner.dart';
import 'package:io/ansi.dart' as io;

const _outputName = 'strings.json';

class GenStringsCommand extends Command<int> {
  GenStringsCommand() {
    argParser.addOption('output',
        abbr: 'o',
        help: 'your output directory path -- default ./lib/generated.');
    argParser.addOption('assets',
        abbr: 'a', help: 'your asset directory path -- default ./lib');
  }
  @override
  String get description => 'Auto generate chinese strings to json file';

  @override
  String get name => 'gen-strings';

  Logger logger = Logger.standard();

  Future<int> run() async {
    String outputPath = './lib/generated';
    String libPath = './lib';

    if (argResults['output'] != null) {
      outputPath = argResults['output'];
    }
    if (argResults['assets'] != null) {
      libPath = argResults['assets'];
    }
    logger.stdout('tip: assets directory path in `${libPath}`');
    logger.stdout('tip: output directory path in `${outputPath}`');

    Directory libDirectory = Directory(libPath);
    if (!await libDirectory.exists()) {
      logger.stdout(
          io.wrapWith('error: assets file path is not exists.', [io.red]));
      print('exit 0');
      return 0;
    }
    Progress progress =
        logger.progress('waiting: seek chinese strings form your $libPath');
    File stringFile = File('$outputPath/$_outputName');
    if (stringFile.existsSync()) {
      await stringFile.delete();
    }
    await stringFile.create(recursive: true);
    IOSink sink = stringFile.openWrite(mode: FileMode.append);
    List<String> strings = [];
    await readDirectory(libDirectory, strings);
    Map<String, String> resultMap = {};
    for (int i = 0; i < strings.length; i++) {
      if (!resultMap.containsValue(strings[i])) {
        resultMap['$i'] = strings[i];
      }
    }
    sink.write(json.encode(resultMap));
    await sink.close();
    progress.finish(showTiming: true);
    logger.stdout(io.wrapWith(
        'success: generate complete,new file in `${stringFile.path}`',
        [io.green]));
    logger.stdout('All done exit 0');
    return 0;
  }

  Future<void> readDirectory(Directory directory, List<String> strings) async {
    final libList = directory.listSync();
    for (final file in libList) {
      // print('scan:${file.path}');
      if (await FileSystemEntity.isDirectory(file.path)) {
        // assetDirectories.add(file.path);
        await readDirectory(Directory(file.path), strings);
      } else {
        File mFile = File(file.path);
        String sourceContent = await mFile.readAsString();
        int contentLength = sourceContent.length;

        final _scanner = StringScanner(sourceContent);
        int lastLootPosition = _scanner.position;

        void collectionString(
          int start,
          int end,
        ) {
          final resultContent = sourceContent.substring(start, end);
          if (resultContent.isNotEmpty &&
              RegExp('[\u4e00-\u9fa5]').hasMatch(resultContent)) {
            // String transformResult = await transform(con);
            // print(con);
            // print(resultContent);
            strings.add(resultContent);
          }
        }

        while (!_scanner.isDone) {
          _scanner.scan(RegExp(r'\s+'));

          if (_scanner.scan(RegExp(r'"""(?:[^"\\]|\\(.|\n))*"""')) ||
              _scanner.scan(RegExp(r"'''(?:[^'\\]|\\(.|\n))*'''"))) {
            final ignoreIndex = 3;
            collectionString(_scanner.lastMatch.start + ignoreIndex,
                _scanner.lastMatch.end - ignoreIndex);
            continue;
          }

          if (_scanner.scan(RegExp(r'r"""(?:[^"\\]|\\(.|\n))*"""')) ||
              _scanner.scan(RegExp(r"r'''(?:[^'\\]|\\(.|\n))*'''"))) {
            final ignoreIndex = 3;
            collectionString(_scanner.lastMatch.start + ignoreIndex+1,
                _scanner.lastMatch.end - ignoreIndex);
            continue;
          }

          if (_scanner.scan(RegExp(r'".*"')) ||
              _scanner.scan(RegExp(r"'.*'"))) {
            final ignoreIndex = 1;
            collectionString(_scanner.lastMatch.start + ignoreIndex,
                _scanner.lastMatch.end - ignoreIndex);
            continue;
          }

          if (_scanner.scan(RegExp(r'r".*"')) ||
              _scanner.scan(RegExp(r"r'.*'"))) {
            final ignoreIndex = 1;
            collectionString(_scanner.lastMatch.start + ignoreIndex+1,
                _scanner.lastMatch.end - ignoreIndex);
            continue;
          }

          if (_scanner.position == contentLength) {
            break;
          }
          if (lastLootPosition == _scanner.position) {
            _scanner.position = lastLootPosition + 1;
          }
          lastLootPosition = _scanner.position;
        }
      }
    }
  }
}
